#include<stdio.h>
#include<string.h>

//const = empêche la modification de la variable dans la fonction
//Qu'on lui donne une chaîne constante ou non
//Evite de faire des bêtises et donne une indication sur le fonctionnement
//Rappel : à ce stade (prototype des fonctions), pas de différences entre
//char *dst et char dst[] (en effet les tableaux n'existent pas vraiment),
//on manipule des pointeurs

//Copie la chaine source dans la destination.
//Retourne un pointeur sur le 0 terminal de la destination.
char* my_stpcpy(char dst[], const char *src) {
  int len = strlen(src);
  for (int i = 0; i <= len; i++) {
    dst[i] = src[i];
  }
  return dst + len;
}

//Version notation pointeur (plus condensée)
char *my_stpcpy_ptr(char *dst, const char *src) {

  while(*src)
      *dst++ = *src++;
  *dst = '\0'; //Recopie du zéro terminal
  return dst;
}

//Concatène la chaine s2 à la chaine s1 et retourne un pointeur sur s1.
//On considère s1 suffisamment large pour accueillir s2
char * my_strcat(char *s1, const char*s2) {
  int len = strlen(s1);
  int i = 0;
  while (s2[i]) {
    s1[len+i] = s2[i];
    i++;
  }
  s1[len+i] = '\0';
  return s1;
}

char * my_strcat_ptr(char *s1, const char*s2) {
  char *tmp = s1;
  while(*s1)
    s1++;
  my_stpcpy_ptr(s1, s2);
  return tmp;
}

//Compare les chaines s1 et s2 et retourne un nombre négatif si s1 est avant s2 dans
//l'ordre lexicographique, positif si s1 est après s2, zéro s'il s’agit des
//mêmes chaines.
int my_strcmp(const char *s1, const char *s2) {

  while(*s1) { //Tant qu'on lit dans s1, càd qu'on ne trouve pas le zéro terminal
    if (*s2 == 0)  //Si s2 est finie, càd on est sur le zéro terminal
      return -1;  //Sort directement de la fonction et renvoie -1
    if (*s1 != *s2) //Si on a une différence on renvoie l'écart de valeur des caractères
      return *s1 - *s2;
    s1++;
    s2++;
  }
  // On regarde si s2 est finie aussi.
  //Si la caractère pointé par s2 est différent de 0, alors les chaînes sont différentes, alors s2 plus longue, on renvoie 1
  //Si s2 est finie aussi, les chaînes sont équivalentes, on renvoie 0
  return *s2 != 0;

}


int my_strlen_ptr(char *str) {
  int len = 0;
  while(*str++)
    len++;
  return len;
}


int main(int argc, char*argv[]) {

  //char *tutu = "salut";  = chaîne constante (cf. slide 57)
  //tutu pointeur vers la constante salut, pas le droit de modifier
  //Comme si un const était caché - équivalent à : const char* tutu = "salut";
  //mais ça ne vous le dit pas, donc vous n'avez aucun warning quand
  //vous la manipulez comme une chaîne de caractères normale
  //Si on avait déclaré : const char str[] = "salut";
  //on aurait eu la même chose, une chaîne constante.

  //Déclaration de chaînes de caractères licites
  char toto[] = "salutcdscsdc";
  char titi[] = {'s','o','l','u','t','\0'};

  //On fait attention à avoir alloué assez de place dans toto pour recevoir titi
  printf("STRCPY\n");
  printf("Chaîne dest : %s, chaîne source : %s\n", toto, titi);
  char *end_toto = my_stpcpy_ptr(toto, titi);  //!!! prévoir assez d'espace dans toto pour mettre titi
  printf("Adresse de départ : %p - adresse de fin renvoyée par my_strcpy : %p\n", toto, end_toto); //affichage adresse de départ et adresse terminale
  printf("%s\n", toto);

  printf("\nSTRCAT\n");
  //Alloue un large tableau de 50 caractères, initialisé à zéro,
  //et dont les premiers éléments définissent une châine de caractères
  char gro_toto[50] = {'s','i','l','u','t','\0'};
  printf("chaine 1 : %s chaine 2 : %s\n", gro_toto, "hello");
  my_strcat_ptr(gro_toto, "hello"); //! "hello" est une constante ainsi
  printf("après cat : %s\n", gro_toto);

  printf("\nSTRCMP\n");
  char test[] = "aolutdscdc";
  printf("%s, %s\n", test, titi);
  printf("my strcmp : %d, strcmp : %d\n", my_strcmp(test,titi), strcmp(test,titi));



  return 0;

}
